$ErrorActionPreference = 'Stop';
$toolsDir    = "$(Split-Path -parent $MyInvocation.MyCommand.Definition)"
$unzPath = "${env:TEMP}"
$osBitness = Get-ProcessorBits
if(Test-Path Env:\ChocolateyPackageFolder) {
  . $env:ChocolateyPackageFolder\tools\chocolateyfunctions.ps1
  . $env:ChocolateyPackageFolder\tools\chocolateyvariables.ps1
} else {
  . ((Get-Item -Path ".\" -Verbose).FullName + "\tools\chocolateyfunctions.ps1")
  . ((Get-Item -Path ".\" -Verbose).FullName + "\tools\chocolateyvariables.ps1")
}

$pp = Get-PackageParameters
if ($pp["NoVst2x86"]      -eq $null -or $pp["NoVst2x86"] -eq '')     { $pp["NoVst2x86"] = $false }
if ($pp["NoVst2x64"]      -eq $null -or $pp["NoVst2x64"] -eq '')     { if($osBitness -eq 64) { $pp["NoVst2x64"] = $false }}
if ($pp["NoVst3x86"]      -eq $null -or $pp["NoVst3x86"] -eq '')     { $pp["NoVst3x86"] = $false }
if ($pp["NoVst3x64"]      -eq $null -or $pp["NoVst3x64"] -eq '')     { if($osBitness -eq 64) { $pp["NoVst3x64"] = $false }}
if ($pp["NoAaxx86"]       -eq $null -or $pp["NoAaxx86"] -eq '')      { $pp["NoAaxx86"] = $false }
if ($pp["NoAaxx64"]       -eq $null -or $pp["NoAaxx64"] -eq '')      { if($osBitness -eq 64) { $pp["NoAaxx64"] = $false }}
if ($pp["NoPresets"]      -eq $null -or $pp["NoPresets"] -eq '')     { $pp["NoPresets"] = $false }
if ($pp["NoUserFolder"]   -eq $null -or $pp["NoUserFolder"] -eq '')  { $pp["NoUserFolder"] = $false }
if ($pp["NoShortcuts"]    -eq $null -or $pp["NoShortcuts"] -eq '')   { $pp["NoShortcuts"] = $false }
if ($pp["InstallerPath"]  -eq $null -or $pp["InstallerPath"] -eq '') { $fileLocation = "$unzPath\$unzInstPath"; $pp["InstallerPath"]=$false } else { $fileLocation = $pp["InstallerPath"] }
if ($pp["CompanyPath"]    -eq $null -or $pp["CompanyPath"] -eq '')   { $pp["CompanyPath"] = $false }                                          else { $standardCompanyPath=$companyPath; $companyPath = $pp["CompanyPath"] }
if ($pp["LibraryPath"]    -eq $null -or $pp["LibraryPath"] -eq '')   { $pp["LibraryPath"] = $false }                                          else { $libraryPath = $pp["LibraryPath"] }
if ($pp["UserFolderPath"] -eq $null -or $pp["UserFolderPath"] -eq ''){ }                                                                      else { $userFolderPath = $pp["UserFolderPath"]}
if ($pp["Vst2Path"]       -eq $null -or $pp["Vst2Path"] -eq '')      { }                                                                      else { $vst2Path = $pp["Vst2Path"] }
if ($pp["Vst2x86Path"]    -eq $null -or $pp["Vst2x86Path"] -eq '')   { }                                                                      else { $vst2x86_64Path = $pp["Vst2x86Path"] }

# Create/Overwrite bit-aware variables
CreateBitAwareVariables

#Create objects after package parameters changed standard values
CreateRegistryObjects
CreateRegistryFileObjects
CreateShortcutObjects
CreateSymlinkObjects
CreateInstallerObjects
CreatePackageRessourcePathObjects
CreateTxtFileObjects
CreatePackageParametersObjects

$objectsLists = $regKeys, $regKeyFileObjects, $shortcuts, $symlinks, $installerComponentsList, $PackageRessourcePathList, $PackageNewFiles
Foreach($objectsList in $objectsLists) {
  Foreach($object in $objectsList) {
    $object.Add('execute', $false)
    $bitStatus = ($object.bit -eq $null) -Or ($object.bit -Contains $osBitness)
    $ppIndependent = (($object.validpp -eq $null) -Or ($object.validpp -Like "Always"))
    Write-Debug ("ppindependent: " + $ppIndependent)
    Write-Debug ("bitStatus: " + $bitStatus)

    Foreach($packageParameter in $object.validpp) {
      Write-Debug ("Status of evaluated package parameter:" + $packageParameter)
      if((Test-Path variable:pp.packageParameter) -eq $true) {
        Write-Debug $pp[$packageParameter]
        Write-Debug ($pp[$packageParameter] -eq $false)
      }
      if(($pp[$packageParameter] -eq $false) -And $bitStatus) {
        $object.execute = $true
      }
    }

    if($ppIndependent -And $bitStatus) {
      $object.execute = $true
    }
  }
}

Write-Debug ("This is the x86 installation path: " + $vst2x86_64Path)
Write-Debug ("Company Path: " + $companyPath)

# Create nupkg file without embedded files
Write-Debug ("This should be the package title: " + (".\" + $env:ChocolateyPackageName + ".nupkg"))
.$env:ChocolateyInstall\tools\7z.exe d ($env:ChocolateyPackageFolder + "\" + $env:ChocolateyPackageName + ".nupkg")  * -r  -xr!package -xr!tools -xr!_rels -x!"*.nuspec" -x!"[Content_Types].xml"

# Create empty uninstall file
Out-File -FilePath ($env:ChocolateyPackageFolder + "\uninstall.txt")

Foreach ($item in $regKeys)                  { CreateRegKey($item)              }
Foreach ($item in $regKeyFileObjects)        { CreateRegKeyFromFile($item)      }
Foreach ($item in $shortcuts)                { OverwriteShortcut($item)         }
Foreach ($item in $symlinks)                 { OverwriteSymlink ($item)         }
Foreach ($item in $PackageRessourcePathList) { CopyPackageRessources($item)     }
Foreach ($item in $PackageRessourcePathList) { WriteUninstallData($item)        }
Foreach ($item in $PackageRessourcePathList) { RemoveTemporaryFiles($item)      }
Foreach ($item in $PackageNewFiles)          { CreateTxtFiles($item)            }
Foreach ($item in $installerComponentsList)  { CreateInstallerParameters($item) }
Foreach ($item in $packageParametersObjectsList) { HandlePackageArgs($item)     }
Foreach ($item in $packageParametersObjectsList) { RunInstallerWithPackageParametersObject($item) }
Foreach ($item in $packageParametersObjectsList) { RemoveInstallerObjects($item) }
